<?php
/**
 * Created by PhpStorm.
 * User: jderosa
 * Date: 21/02/20
 * Time: 11.57
 */

namespace FiloBlu\Rma\Setup;


use Magento\Framework\Setup\UpgradeSchemaInterface;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\SchemaSetupInterface;

class UpgradeSchema implements UpgradeSchemaInterface
{

    const EASY_RETURN_MATRIXRATE_TABLE = 'filoblu_easy_return_matrixrate';

    public function upgrade( SchemaSetupInterface $setup, ModuleContextInterface $context ) {

        $setup->startSetup();

        if (!$setup->tableExists(\FiloBlu\Rma\Api\RmaExtensionAttributesRepositoryInterface::TABLE_NAME)) {
           $this->createFiloBluRmaItemTable($setup);
        }

        if (!$setup->tableExists(\FiloBlu\Rma\Model\ResourceModel\EasyReturn::TABLE_NAME)) {
            $this->createEasyReturnTable($setup);
        }

        if (!$setup->tableExists(self::EASY_RETURN_MATRIXRATE_TABLE)) {
            $this->createReturnMatrixRateTable($setup);
        }

        $setup->endSetup();

    }


    /**
     * @param SchemaSetupInterface $setup
     * @throws \Zend_Db_Exception
     */
    protected function createFiloBluRmaItemTable(SchemaSetupInterface $setup){

        $table =   $setup->getConnection()
            ->newTable($setup->getConnection()->getTableName(\FiloBlu\Rma\Api\RmaExtensionAttributesRepositoryInterface::TABLE_NAME))
            ->addColumn(
                'id',
                \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                null,
                ['identity' => true, 'unsigned' => true, 'nullable' => false, 'primary' => true],
                'id')
            ->addColumn('rma_item_id',
                \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                null,
                ['unsigned' => true, 'nullable' => false],
                'rma_entity_id')
            ->addColumn('order_id',
                \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                null,
                ['unsigned' => true, 'nullable' => false],
                'order_id');
        $setup->getConnection()->createTable($table);

    }

    /**
     * @param SchemaSetupInterface $setup
     * @throws \Zend_Db_Exception
     */
    protected function createEasyReturnTable(SchemaSetupInterface $setup){
        $tableName = $setup->getConnection()->getTableName(\FiloBlu\Rma\Model\ResourceModel\EasyReturn::TABLE_NAME);
        $parentTableName = $setup->getConnection()->getTableName('magento_rma');
        $table =   $setup->getConnection()
            ->newTable($tableName)
            ->addColumn(
                'id',
                \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                null,
                ['identity' => true, 'unsigned' => true, 'nullable' => false, 'primary' => true],
                'id')
            ->addColumn('rma_entity_id',
                \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                null,
                ['unsigned' => true, 'nullable' => false],
                'rma_entity_id')
            ->addColumn('customer_firstname',
                \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                null,
                ['nullable' => false],
                'customer_firstname')
            ->addColumn('customer_lastname',
                \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                null,
                ['nullable' => false],
                'customer_lastname')
            ->addColumn('street',
                \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                null,
                ['nullable' => false],
                'street')
            ->addColumn('city',
                \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                null,
                ['nullable' => false],
                'city')
            ->addColumn('region_id',
                \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                null,
                ['nullable' => false],
                'region_id')
            ->addColumn('region',
                \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                null,
                ['nullable' => false],
                'region')
            ->addColumn('postcode',
                \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                null,
                ['nullable' => false],
                'postcode')
            ->addColumn('country_id',
                \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                null,
                ['nullable' => false],
                'country_id')
            ->addColumn('theoric_shipping_amount',
                \Magento\Framework\DB\Ddl\Table::TYPE_FLOAT,
                null,
                ['nullable' => false],
                'theoric_shipping_amount')
            ->addIndex($setup->getIdxName(
                $tableName,['rma_entity_id'],\Magento\Framework\DB\Adapter\AdapterInterface::INDEX_TYPE_UNIQUE),
                ['rma_entity_id'],['type' => \Magento\Framework\DB\Adapter\AdapterInterface::INDEX_TYPE_UNIQUE])
            ->addForeignKey(
                $setup->getFkName($tableName,'rma_entity_id',$parentTableName,'entity_id'),
                'rma_entity_id',
                $setup->getTable($parentTableName),
                'entity_id',
                \Magento\Framework\DB\Ddl\Table::ACTION_CASCADE);

        $setup->getConnection()->createTable($table);
    }


    /**
     * @param SchemaSetupInterface $setup
     * @throws \Zend_Db_Exception
     */
    protected function createReturnMatrixRateTable(SchemaSetupInterface $setup){
        /**
         * Create table 'filoblu_easy_return_matrixrate'
         */
        $table = $setup->getConnection()->newTable(
            $setup->getTable('filoblu_easy_return_matrixrate')
        )->addColumn(
            'pk',
            \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
            null,
            ['identity' => true, 'unsigned' => true, 'nullable' => false, 'primary' => true],
            'Primary key'
        )->addColumn(
            'website_id',
            \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
            null,
            ['nullable' => false, 'default' => '0'],
            'Website Id'
        )->addColumn(
            'dest_country_id',
            \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
            4,
            ['nullable' => false, 'default' => '0'],
            'Destination coutry ISO/2 or ISO/3 code'
        )->addColumn(
            'dest_region_id',
            \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
            null,
            ['nullable' => false, 'default' => '0'],
            'Destination Region Id'
        )->addColumn(
            'dest_city',
            \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
            30,
            ['nullable' => false, 'default' => ''],
            'Destination City'
        )->addColumn(
            'dest_zip',
            \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
            10,
            ['nullable' => false, 'default' => '*'],
            'Destination Post Code (Zip)'
        )->addColumn(
            'dest_zip_to',
            \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
            10,
            ['nullable' => false, 'default' => '*'],
            'Destination Post Code To (Zip)'
        )->addColumn(
            'condition_name',
            \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
            20,
            ['nullable' => false],
            'Rate Condition name'
        )->addColumn(
            'condition_from_value',
            \Magento\Framework\DB\Ddl\Table::TYPE_DECIMAL,
            '12,4',
            ['nullable' => false, 'default' => '0.0000'],
            'Rate condition from value'
        )->addColumn(
            'condition_to_value',
            \Magento\Framework\DB\Ddl\Table::TYPE_DECIMAL,
            '12,4',
            ['nullable' => false, 'default' => '0.0000'],
            'Rate condition to value'
        )->addColumn(
            'price',
            \Magento\Framework\DB\Ddl\Table::TYPE_DECIMAL,
            '12,4',
            ['nullable' => false, 'default' => '0.0000'],
            'Price'
        )->addColumn(
            'cost',
            \Magento\Framework\DB\Ddl\Table::TYPE_DECIMAL,
            '12,4',
            ['nullable' => false, 'default' => '0.0000'],
            'Cost'
        )->addColumn(
            'shipping_method',
            \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
            255,
            ['nullable' => false],
            'Shipping Method'
        )->addIndex(
            $setup->getIdxName(
                'filoblu_easy_return_matrixrate',
                ['website_id', 'dest_country_id', 'dest_region_id', 'dest_city', 'dest_zip', 'condition_name',
                    'condition_from_value','condition_to_value','shipping_method'],
                \Magento\Framework\DB\Adapter\AdapterInterface::INDEX_TYPE_UNIQUE
            ),
            ['website_id', 'dest_country_id', 'dest_region_id', 'dest_city', 'dest_zip', 'condition_name',
                'condition_from_value','condition_to_value','shipping_method'],
            ['type' => \Magento\Framework\DB\Adapter\AdapterInterface::INDEX_TYPE_UNIQUE]
        )->setComment(
            'FiloBlu Easy Return Shipping MatrixRate'
        );
        $setup->getConnection()->createTable($table);

    }

}